//package com.bonus.material.lease.controller;
//
//import java.util.List;
//import javax.servlet.http.HttpServletResponse;
//import com.bonus.common.log.enums.OperaType;
//import com.bonus.material.common.annotation.PreventRepeatSubmit;
//import io.swagger.annotations.Api;
//import io.swagger.annotations.ApiOperation;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.web.bind.annotation.GetMapping;
//import org.springframework.web.bind.annotation.PostMapping;
//import org.springframework.web.bind.annotation.PutMapping;
//import org.springframework.web.bind.annotation.DeleteMapping;
//import org.springframework.web.bind.annotation.PathVariable;
//import org.springframework.web.bind.annotation.RequestBody;
//import org.springframework.web.bind.annotation.RequestMapping;
//import org.springframework.web.bind.annotation.RestController;
//import com.bonus.common.log.annotation.SysLog;
//import com.bonus.material.lease.domain.LeaseOutDetails;
//import com.bonus.material.lease.service.ILeaseOutDetailsService;
//import com.bonus.common.core.web.controller.BaseController;
//import com.bonus.common.core.web.domain.AjaxResult;
//import com.bonus.common.core.utils.poi.ExcelUtil;
//import com.bonus.common.core.web.page.TableDataInfo;
//
///**
// * 领料出库详细Controller
// *
// * @author xsheng
// * @date 2024-10-16
// */
//@Api(tags = "领料出库详细接口")
//@RestController
//@RequestMapping("/lease_out_details")
//public class LeaseOutDetailsController extends BaseController {
//
//    @Autowired
//    private ILeaseOutDetailsService leaseOutDetailsService;
//
//    /**
//     * 查询领料出库详细列表
//     */
//    @ApiOperation(value = "查询领料出库详细列表")
//    @GetMapping("/list")
//    public TableDataInfo list(LeaseOutDetails leaseOutDetails) {
//        startPage();
//        List<LeaseOutDetails> list = leaseOutDetailsService.selectLeaseOutDetailsList(leaseOutDetails);
//        return getDataTable(list);
//    }
//
//    /**
//     * 导出领料出库详细列表
//     */
//    @ApiOperation(value = "导出领料出库详细列表")
//    @PreventRepeatSubmit
//    @SysLog(title = "领料出库详细", businessType = OperaType.EXPORT, logType = 1,module = "仓储管理->导出领料出库详细")
//    @PostMapping("/export")
//    public void export(HttpServletResponse response, LeaseOutDetails leaseOutDetails) {
//        List<LeaseOutDetails> list = leaseOutDetailsService.selectLeaseOutDetailsList(leaseOutDetails);
//        ExcelUtil<LeaseOutDetails> util = new ExcelUtil<>(LeaseOutDetails.class);
//        util.exportExcel(response, list, "领料出库详细数据");
//    }
//
//    /**
//     * 获取领料出库详细详细信息
//     */
//    @ApiOperation(value = "获取领料出库详细详细信息")
//    @GetMapping(value = "/{id}")
//    public AjaxResult getInfo(@PathVariable("id") Long id) {
//        return success(leaseOutDetailsService.selectLeaseOutDetailsById(id));
//    }
//
//    /**
//     * 新增领料出库详细
//     */
//    @ApiOperation(value = "新增领料出库详细")
//    @PreventRepeatSubmit
//    @SysLog(title = "领料出库详细", businessType = OperaType.INSERT, logType = 1,module = "仓储管理->新增领料出库详细")
//    @PostMapping
//    public AjaxResult add(@RequestBody List<LeaseOutDetails> leaseOutDetailsList) {
//        try {
//            return leaseOutDetailsService.insertLeaseOutDetails(leaseOutDetailsList);
//        } catch (Exception e) {
//            return error("系统错误, " + e.getMessage());
//        }
//    }
//
//    /**
//     * 修改领料出库详细
//     */
//    @ApiOperation(value = "修改领料出库详细")
//    @PreventRepeatSubmit
//    @SysLog(title = "领料出库详细", businessType = OperaType.UPDATE, logType = 1,module = "仓储管理->修改领料出库详细")
//    @PutMapping
//    public AjaxResult edit(@RequestBody LeaseOutDetails leaseOutDetails) {
//        try {
//            return toAjax(leaseOutDetailsService.updateLeaseOutDetails(leaseOutDetails));
//        } catch (Exception e) {
//            return error("系统错误, " + e.getMessage());
//        }
//    }
//
//    /**
//     * 删除领料出库详细
//     */
//    @ApiOperation(value = "删除领料出库详细")
//    @PreventRepeatSubmit
//    @SysLog(title = "领料出库详细", businessType = OperaType.DELETE, logType = 1,module = "仓储管理->删除领料出库详细")
//	@DeleteMapping("/{ids}")
//    public AjaxResult remove(@PathVariable Long[] ids) {
//        return toAjax(leaseOutDetailsService.deleteLeaseOutDetailsByIds(ids));
//    }
//}
